import Parser from "rss-parser/index";
import { ListenerRSSInfos as ListenerInfo } from "./Models/ListenerRSSInfos";

const DEFAULT_TIMELOOP: number = 5 * 60; // default timeloop is 5 min

export class ListenerRss {
  name: string = "";
  address: string = "";
  timeloop: number = DEFAULT_TIMELOOP; // time in seconds
  customfields?: [string, string | [string, string]][];

  // private fields
  parser: Parser | undefined = undefined;
  loopRunning: boolean = false;

  constructor(config: ListenerInfo) {
    this.setData(config);
    this.setParser();
  }

  setParser() {
    // set parser
    this.parser = new Parser(
      this.customfields !== undefined
        ? {
            customFields: {
              item: this.customfields.map((elt) => {
                return Array.isArray(elt[1]) ? elt[1][0] : elt[1];
              }),
            },
          }
        : {}
    ); // if customfield is set -> let's set the parser with, else let the option empty
  }

  setData(infos: ListenerInfo) {
    // Set data
    this.name = infos.name;
    this.address = infos.address;
    this.timeloop =
      infos.timeloop === undefined ? DEFAULT_TIMELOOP : infos.timeloop;
    this.customfields =
      infos.customfields === undefined ? undefined : infos.customfields;
  }

  fetchRSS(): any {
    // TODO Pas Bien
    if (this.parser !== undefined && this.address !== undefined) {
      return this.parser.parseURL(this.address).catch((err) => {
        throw new Error("bad address or no access : " + err);
      });
    } else throw new Error("listener must be first initialized");
  }

  /**
   * @brief call the callback function each looptime
   * @param callback function who's going to be called with the latest get
   */
  start(callback: any) {
    // TODO any = Pas bien !!
    /**
     * Un des soucis qu'on a c'est sur le typage de l'objet de retour. le problème étant que la nature de l'obj de
     * retour ne peut pas etre connue puisque il depend des custom fields qui sont definit par l'utilisateur. L'idée
     * pourrait etre de creer une classe generique (cf le type CustomFields du ficher index.d.ts du package rss-parser).
     * Après quelques recherches ca doit etre la manière ""correct"" de faire. Entre autre avec l'utilisation des mots
     * clés keyof U ou encore Array<keyof U>. Je vais continuer a gratter dans cette direction perso.
     */
    this.loopRunning = true;

    (async () => {
      while (this.loopRunning) {
        this.fetchRSS().then((obj: any, err: any) => callback(obj, err)); // TODO Erreur a la compile
        await new Promise((res) => setTimeout(res, 2000));
      }
    })();
  }

  /**
   * @brief stop the async function
   */
  stop() {
    this.loopRunning = false;
  }
}
